-- SECTION 1 --
-- The following queries are used for creating and setting up an Oracle database if you need to create it from scratch.
-- This process is complicated and requires connecting to your Oracle instance in a special manner.
-- See the "Creating a Database with the CREATE DATABASE Statement" section in your Oracle database documentation for details.
-- This can be found at https://docs.oracle.com/database/121/ADMIN/create.htm#ADMIN-GUID-18B03451-5C74-4B53-A892-656C3E8A2556 for Oracle 12.1.
-- If you have a DBA, it is highly recommended to let them handle this step.
-- Please skip this section if you are using Oracle instance on the cloud

-- Set this to what you'd like to name your database. This must be 8 characters or less.
DEFINE DBNAME = "FDS";

-- Set this to the password for the SYS administrator user for your database. Double quote characters are not allowed.
DEFINE SYS_PASSWORD = "change_on_install";

-- Set this to the password for the SYSTEM administrator user for your database. Double quote characters are not allowed.
DEFINE SYSTEM_PASSWORD = "manager";

-- Set this to the name you want to give the default tablespace for your database. This must be 30 characters or less.
DEFINE DEF_TBSPACE = "LOADERTBS";

-- Set this to the name that will be used for the first datafile for the database.
DEFINE DATA_FILE_NAME = "fds1.dbf";

-- Set this to the size for the first datafile for the database. Since "AUTOEXTEND" is used in the query below this does not need to be the size of your entire subscription.
-- If your subscription is quite small this can be decreased.
DEFINE DATA_FILE_SIZE = "256M";

-- To print the CREATE DATABASE query that will be executed (with substitutions made) without actually executing it, run the following query:

SELECT 'CREATE DATABASE &DBNAME
USER SYS IDENTIFIED BY "&SYS_PASSWORD"
USER SYSTEM IDENTIFIED BY "&SYSTEM_PASSWORD"
CHARACTER SET AL32UTF8
NATIONAL CHARACTER SET AL16UTF16
EXTENT MANAGEMENT LOCAL
DEFAULT TABLESPACE &DEF_TBSPACE
    DATAFILE ''&DATA_FILE_NAME'' SIZE &DATA_FILE_SIZE REUSE AUTOEXTEND ON MAXSIZE UNLIMITED
    EXTENT MANAGEMENT LOCAL;' FROM DUAL;

-- Here is the CREATE DATABASE query to execute:
CREATE DATABASE &DBNAME
USER SYS IDENTIFIED BY "&SYS_PASSWORD"
USER SYSTEM IDENTIFIED BY "&SYSTEM_PASSWORD"
CHARACTER SET AL32UTF8
NATIONAL CHARACTER SET AL16UTF16
EXTENT MANAGEMENT LOCAL
DEFAULT TABLESPACE &DEF_TBSPACE
    DATAFILE '&DATA_FILE_NAME' SIZE &DATA_FILE_SIZE REUSE AUTOEXTEND ON MAXSIZE UNLIMITED
    EXTENT MANAGEMENT LOCAL;

-- SECTION 2 --
-- The following queries are used for creating and setting up a user once you already have an Oracle database created.
-- Connect to the database, change the parameters below as desired, and then run the queries.
-- If you have a DBA, they may need to run these queries for you.

SET SERVEROUTPUT ON SIZE UNLIMITED;

DECLARE
  -- Set this to the name of the user.
  LoaderUserName VARCHAR2(30) := 'FDS_USER';

  -- Set this to the user's password. Escape single quotes as '' (two single quote characters). Double quote characters are not allowed.
  LoaderUserPassword VARCHAR2(50) := 'admin';

  -- Set this to the tablespace where you'd like the Loader to store tables. This will be the default tablespace for the "LoaderUserName".
  -- If you did not set up the Oracle database yourself, consult with your DBA to determine what this should be set to.
  -- Alternatively, if you know you want to use the same default tablespace as a user that already exists, you can run one of the following queries to determine what to enter here:
     -- SELECT DEFAULT_TABLESPACE FROM USER_USERS; (as the user whose default tablespace you want to match)
     -- SELECT DEFAULT_TABLESPACE FROM DBA_USERS WHERE USERNAME = '<USER_TO_FIND_DEFTBS_FOR>'; (as an Oracle DBA user)
  DefaultTablespace VARCHAR2(30) := 'LOADERTBS';
  DefaultDatafileSize VARCHAR2(30) := '256M';
  
  -- Set this to 1 if you are using Oracle instance on the cloud and you want to create a new tablespace for the SDF content, set to 0 otherwise.
  -- For putting in file path make sure you quote your path i.e. '''/datafile.dbf'''(you will need to use two single quotes to escapte the quote)
  CreateTablespace VARCHAR2(1) := '1';
  -- Set this to the database file you want create, if you are using Oracle instance on the cloud or have Oracle Managed Files turned on please leave this empty.
  DataFilePath VARCHAR2(256) := '';
  -- Set this to 1 if you would like the procedure to generate and execute the script; leave it set to 0 if you would like the procedure to just print the script.
  ExecuteSql VARCHAR2(1) := '0';
  
 
  create_tablespace_query VARCHAR2(32767);
  create_user_query VARCHAR2(32767);
  grant_create_session_query VARCHAR2(32767);
  grant_create_table_query VARCHAR2(32767);
  
BEGIN
  create_tablespace_query := 'CREATE TABLESPACE ' || DefaultTablespace || ' DATAFILE ' || DataFilePath || ' SIZE ' || DefaultDatafileSize || ' AUTOEXTEND ON MAXSIZE UNLIMITED';
  create_user_query := 'CREATE USER ' || LoaderUserName || ' IDENTIFIED BY "' || LoaderUserPassword || '" DEFAULT TABLESPACE ' || DefaultTablespace || ' QUOTA UNLIMITED ON ' || DefaultTablespace;
  grant_create_session_query := 'GRANT CREATE SESSION TO ' || LoaderUserName;
  grant_create_table_query := 'GRANT CREATE TABLE TO ' || LoaderUserName;
  IF ExecuteSql = '0' THEN
    IF CreateTablespace = '1' THEN
        DBMS_OUTPUT.PUT_LINE(create_tablespace_query || ';');
    END IF;
    DBMS_OUTPUT.PUT_LINE(create_user_query || ';');
    DBMS_OUTPUT.PUT_LINE(grant_create_session_query || ';');
    DBMS_OUTPUT.PUT_LINE(grant_create_table_query || ';');
  ELSIF ExecuteSql = '1' THEN
    IF CreateTablespace = '1' THEN
        DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || create_tablespace_query || ';');
        EXECUTE IMMEDIATE create_tablespace_query;
    END IF;
    DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || create_user_query || ';');
    EXECUTE IMMEDIATE create_user_query;
    DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || grant_create_session_query || ';');
    EXECUTE IMMEDIATE grant_create_session_query;
    DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || grant_create_table_query || ';');
    EXECUTE IMMEDIATE grant_create_table_query;
  END IF;
END;

-- SECTION 3 --
-- The following queries are used for setting up a directory object once you already have an Oracle database and user created.
-- Connect to the database, change the parameters below as necessary, and then run the queries.
-- If you have a DBA, they may need to run these queries for you.
-- After running these queries, you can see the newly created directory object using the query: SELECT * FROM ALL_DIRECTORIES;
-- Please skip this section if you are using Oracle instance on the cloud

SET SERVEROUTPUT ON SIZE UNLIMITED;

DECLARE
  -- Set this to the name of the user.
  LoaderUserName VARCHAR2(30) := 'FDS_USER';

  -- Set this to the path to the share location from the database server (or the Loader location if the Loader is running directly on the database server).
  -- Please retain a trailing slash (forward slash if database server is Linux, backslash if database server is Windows).
  ShareDrivePathFromDbServer VARCHAR2(512) := '/share/drive/path/from/database/server/';

  -- Set this to the subdirectory of the "ShareDrivePathFromDbServer" you would like the Loader to pass files to Oracle through.
  -- Please specify the same subdirectory during the Loader setup process (FDSLoader64 --setup).
  LoaderSubDir VARCHAR2(64) := 'subdir';

  -- Set this to the name you'd like to use for the Oracle directory object corresponding to the "LoaderSubDir".
  -- Please specify the same directory object name during the Loader setup process (FDSLoader64 --setup).
  DirectoryObjectName VARCHAR2(30) := 'LOADERDIR';

  -- Set this to 1 if you would like the procedure to generate and execute the script; leave it set to 0 if you would like the procedure to just print the script.
  ExecuteSql VARCHAR2(1) := '0';

  create_directory_query VARCHAR2(32767);
  grant_dir_privileges_query VARCHAR2(32767);
BEGIN
  create_directory_query := 'CREATE OR REPLACE DIRECTORY ' || DirectoryObjectName || ' AS ''' || ShareDrivePathFromDbServer || LoaderSubDir || '''';
  grant_dir_privileges_query := 'GRANT ALL ON DIRECTORY ' || DirectoryObjectName || ' TO ' || LoaderUserName;
  IF ExecuteSql = '0' THEN
    DBMS_OUTPUT.PUT_LINE(create_directory_query || ';');
    DBMS_OUTPUT.PUT_LINE(grant_dir_privileges_query || ';');
  ELSIF ExecuteSql = '1' THEN
    DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || create_directory_query || ';');
    EXECUTE IMMEDIATE create_directory_query;
    DBMS_OUTPUT.PUT_LINE('RUNNING QUERY: ' || grant_dir_privileges_query || ';');
    EXECUTE IMMEDIATE grant_dir_privileges_query;
  END IF;
END;
